from .singleton import RedisSingleton
from os import getenv
from typing import Any, List
from datetime import timedelta
from json import dumps, loads

class RedisFacade:
    """
        Fachada de base de datos, crea conexion con un singleton a mssql
    """
    def __init__(self, REDIS_HOST: str = getenv("REDIS_HOST"), REDIS_PORT: int = getenv("REDIS_PORT"), REDIS_PASSWORD: str = getenv("REDIS_PASSWORD")):
        self.redis_manager = RedisSingleton(REDIS_HOST=REDIS_HOST, REDIS_PORT=REDIS_PORT, REDIS_PASSWORD=REDIS_PASSWORD)

    def set_key_value(self, db: int, key: str, value: Any, ttl: timedelta) -> bool: 
        """
            Guarda un valor del tipo key - value de base de datos.
        """       
        
        return self.redis_manager.get_connection(db).set(name=key, value=value, ex=ttl)
        
    def get_key_value(self, db: int, key: str) -> Any: 
        """
            Obtiene un valor del tipo key - value de base de datos.
        """       
        
        return self.redis_manager.get_connection(db).get(name=key).decode("utf-8")
               
    def set_list_str(self, db: int, list_name: str, intems: List[str]) -> bool:
        try:
            for item in intems:
                self.redis_manager.get_connection(db).rpush(list_name, item)    
            return True
        except Exception as e:
            print("Error al almacenar datos de lista:", e)
            return False
            
    def get_list_str(self, db: int, list_name: str) -> List[str]:
        list_data = []
        list_data = self.redis_manager.get_connection(db).lrange(list_name, 0, -1)
        list_data = [item.decode('utf-8') for item in list_data]
        return list_data
        
    def pop_list_str(self, db: int, list_name: str) -> str:
        return self.redis_manager.get_connection(db).lpop(list_name).decode('utf-8')
    
    def len_list(self, db: int, list_name: str) -> str:
        return self.redis_manager.get_connection(db).llen(list_name)
    
    def set_json_data(self, db: int, json_name, json_key, json_data: Any, ttl: timedelta) -> bool:
        is_saved: bool = False
        if isinstance(json_data, dict):
            is_saved = self.redis_manager.get_connection(db).set((json_name + ":" + json_key), dumps(json_data), ex=ttl)
        else:
            is_saved = self.redis_manager.get_connection(db).set((json_name + ":" + json_key), json_data, ex=ttl)
            
        return is_saved
        
    def get_key_json_data(self, db: int, json_name: str, json_key: str) -> Any:
        retrieved_json = self.redis_manager.get_connection(db).get((json_name + ":" + json_key))
        if isinstance(retrieved_json, dict):
            return loads(retrieved_json)  # Convert JSON string back to a Python dictionary
        else:
            return retrieved_json.decode("utf-8")
    
    def get_json_data(self, db: int, json_name: str) -> Any:
        connection = self.redis_manager.get_connection(db)
        data_keys = connection.scan_iter((json_name + ":*"))
        # Recuperar todas las claves
        data = {}
        for key in data_keys:
            retrieved_json = connection.get(key)
            key = key.decode('utf-8').replace((json_name + ":"), "", 1)
            if retrieved_json:  # Verificar si el valor existe
                if isinstance(retrieved_json, dict):
                    data[key] = loads(retrieved_json)
                else:
                    data[key] = retrieved_json.decode('utf-8')

        return data


from redis import Redis, ConnectionError, TimeoutError
from threading import Lock

class RedisSingleton:
    _instance = None
    _lock = Lock()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super().__new__(cls)
                    cls._instance._connections = {}
        return cls._instance
    
    def __init__(self, REDIS_HOST, REDIS_PORT: int, REDIS_PASSWORD: str):
        self.REDIS_HOST = REDIS_HOST
        self.REDIS_PORT = REDIS_PORT
        self.REDIS_PASSWORD = REDIS_PASSWORD

    def get_connection(self, db: int) -> Redis:
        """
        Obtiene una conexión a Redis para una base de datos específica.
        Si la conexión no existe o está dañada, se crea una nueva.
        """
        if db not in self._connections:
            return self._create_connection(db)
        if not self._is_connection_alive(self._connections[db]):
            return self._create_connection(db)
        return self._connections[db]

    def _create_connection(self, db: int) -> Redis:
        """
        Crea una nueva conexión de Redis y la almacena.
        """
        with self._lock:
            self._connections[db] = Redis(
                host=self.REDIS_HOST,  # Cambia según tu configuración
                port=self.REDIS_PORT,
                db=db,
                password=self.REDIS_PASSWORD  # Configura la contraseña si es necesario
            )
        return self._connections[db]

    def _is_connection_alive(self, connection: Redis) -> bool:
        """
        Verifica si la conexión a Redis está viva.
        """
        try:
            # Verifica con un comando básico (PING)
            connection.ping()
            return True
        except (ConnectionError, TimeoutError):
            return False



from setuptools import setup, find_packages

setup(
    name='redisfacade',
    version='1.0.2',
    description='A package to implement a facade for accessing Redis databases',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Alan Medina',
    author_email='amedina.escom@hotmail.com',
    url='https://github.com/amedinag32/redisfacade',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
    install_requires=[
        'redis==5.2.1'
    ],
)
